@@ -4,7 +4,7 @@
 ///        different types if both types are integral
 ///        and sizeof(A) >= sizeof(B).
 ///
-/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2024 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -14,19 +14,17 @@
 #define MIN_HPP
 
 #include <int128_t.hpp>
-
 #include <algorithm>
-#include <type_traits>
 
-namespace primecount {
+namespace {
 
 template <typename A, typename B>
 struct is_comparable
 {
   enum {
-    value = std::is_same<A, B>::value || (
-            prt::is_integral<A>::value &&
-            prt::is_integral<B>::value &&
+    value = port::is_same<A, B>::value || (
+            port::is_integral<A>::value &&
+            port::is_integral<B>::value &&
             sizeof(A) >= sizeof(B))
   };
 };
@@ -35,9 +33,9 @@ template <typename A, typename B, typename C>
 struct is_comparable_3
 {
   enum {
-    value = prt::is_integral<A>::value &&
-            prt::is_integral<B>::value &&
-            prt::is_integral<C>::value &&
+    value = port::is_integral<A>::value &&
+            port::is_integral<B>::value &&
+            port::is_integral<C>::value &&
             sizeof(A) >= sizeof(B) &&
             sizeof(B) >= sizeof(C)
   };
