@@ -18,13 +18,14 @@
 ///         types of CPU cores we try to detect the cache sizes of the
 ///         CPU core type that e.g. occurs most frequently.
 ///
-/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2024 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
 ///
 
 #include <primesieve/CpuInfo.hpp>
+#include <primesieve/macros.hpp>
 
 #include <algorithm>
 #include <stdint.h>
@@ -33,126 +34,18 @@
 #include <string>
 #include <vector>
 
-#if defined(__APPLE__)
-  #if !defined(__has_include)
-    #define APPLE_SYSCTL
-  #elif __has_include(<sys/sysctl.h>)
-    #define APPLE_SYSCTL
-  #endif
+#if defined(__APPLE__) && \
+    __has_include(<sys/sysctl.h>)
+  #define APPLE_SYSCTL
 #endif
 
 #if defined(__i386__) || \
     defined(__x86_64__) || \
     defined(_M_IX86) || \
     defined(_M_X64)
-
-#if defined(_MSC_VER)
-  #include <intrin.h>
-  #include <immintrin.h>
-#endif
-
-#define HAS_CPUID
-
-/* %ebx bit flags */
-#define bit_AVX512F (1 << 16)
-
-/* %ecx bit flags */
-#define bit_AVX512VBMI  (1 << 1)
-#define bit_AVX512VBMI2 (1 << 6)
-#define bit_POPCNT      (1 << 23)
-
-/* xgetbv bit flags */
-#define XSTATE_SSE (1 << 1)
-#define XSTATE_YMM (1 << 2)
-#define XSTATE_ZMM (7 << 5)
-
-namespace {
-
-void run_cpuid(int eax, int ecx, int* abcd)
-{
-#if defined(_MSC_VER)
-  __cpuidex(abcd, eax, ecx);
-#else
-  int ebx = 0;
-  int edx = 0;
-
-  #if defined(__i386__) && \
-      defined(__PIC__)
-    /* in case of PIC under 32-bit EBX cannot be clobbered */
-    __asm__ ("movl %%ebx, %%edi;"
-             "cpuid;"
-             "xchgl %%ebx, %%edi;"
-             : "=D" (ebx),
-               "+a" (eax),
-               "+c" (ecx),
-               "=d" (edx));
-  #else
-    __asm__ ("cpuid;"
-             : "+b" (ebx),
-               "+a" (eax),
-               "+c" (ecx),
-               "=d" (edx));
-  #endif
-
-  abcd[0] = eax;
-  abcd[1] = ebx;
-  abcd[2] = ecx;
-  abcd[3] = edx;
-#endif
-}
-
-// Get Value of Extended Control Register
-int get_xcr0()
-{
-  int xcr0;
-
-#if defined(_MSC_VER)
-  xcr0 = (int) _xgetbv(0);
-#else
-  __asm__ ("xgetbv" : "=a" (xcr0) : "c" (0) : "%edx" );
-#endif
-
-  return xcr0;
-}
-
-bool has_AVX512()
-{
-  int abcd[4];
-
-  run_cpuid(1, 0, abcd);
-
-  // PrimeGenerator::fillNextPrimes() requires POPCNT
-  if ((abcd[2] & bit_POPCNT) != bit_POPCNT)
-    return false;
-
-  int osxsave_mask = (1 << 27);
-
-  // Ensure OS supports extended processor state management
-  if ((abcd[2] & osxsave_mask) != osxsave_mask)
-    return false;
-
-  int ymm_mask = XSTATE_SSE | XSTATE_YMM;
-  int zmm_mask = XSTATE_SSE | XSTATE_YMM | XSTATE_ZMM;
-
-  int xcr0 = get_xcr0();
-
-  // Check AVX OS support
-  if ((xcr0 & ymm_mask) != ymm_mask)
-    return false;
-
-  // Check AVX512 OS support
-  if ((xcr0 & zmm_mask) != zmm_mask)
-    return false;
-
-  run_cpuid(7, 0, abcd);
-
-  // PrimeGenerator::fillNextPrimes() requires AVX512F, AVX512VBMI & AVX512VBMI2
-  return ((abcd[1] & bit_AVX512F) == bit_AVX512F &&
-          (abcd[2] & (bit_AVX512VBMI | bit_AVX512VBMI2)) == (bit_AVX512VBMI | bit_AVX512VBMI2));
-}
-
-} // namespace
-
+  #include <primesieve/cpuid.hpp>
+  #include <primesieve/cpu_supports_avx512_vbmi2.hpp>
+  #define HAS_CPUID
 #endif
 
 #if defined(_WIN32)
@@ -251,8 +144,8 @@ void CpuInfo::init()
       cacheSizes{0, 0, 0, 0},
       cacheSharing{0, 0, 0, 0}
     { }
-    pod_array<size_t, 4> cacheSizes;
-    pod_array<size_t, 4> cacheSharing;
+    Array<size_t, 4> cacheSizes;
+    Array<size_t, 4> cacheSharing;
   };
 
   struct L1CacheStatistics
@@ -862,14 +755,10 @@ std::string CpuInfo::cpuName() const
   }
 }
 
-/// This method is only used by the primesieve command-line app
-/// with the --cpu-info option. Therefore we currently don't
-/// cache the result of has_AVX512().
-///
 bool CpuInfo::hasAVX512() const
 {
   #if defined(HAS_CPUID)
-    return has_AVX512();
+    return cpu_supports_avx512_vbmi2;
   #else
     return false;
   #endif
