@@ -12,7 +12,7 @@
 ///        method, Revista do DETUA, vol. 4, no. 6, March 2006,
 ///        pp. 759-768.
 ///
-/// Copyright (C) 2021 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -21,6 +21,7 @@
 #include <primecount-internal.hpp>
 #include <primesieve.hpp>
 #include <int128_t.hpp>
+#include <macros.hpp>
 #include <min.hpp>
 #include <imath.hpp>
 #include <LoadBalancerP2.hpp>
@@ -28,9 +29,7 @@
 
 #include <stdint.h>
 #include <algorithm>
-#include <cassert>
 
-using namespace std;
 using namespace primecount;
 
 namespace {
@@ -38,17 +37,17 @@ namespace {
 /// Thread sieves [low, high[
 template <typename T>
 T P2_thread(T x,
-           int64_t y,
-           int64_t low,
-           int64_t high)
+            int64_t y,
+            int64_t low,
+            int64_t high)
 {
-  assert(low > 0);
-  assert(low < high);
+  ASSERT(low > 0);
+  ASSERT(low < high);
   int64_t sqrtx = isqrt(x);
   int64_t start = max(y, min(x / high, sqrtx));
   int64_t stop = min(x / low, sqrtx);
-  primesieve::iterator rit(stop + 1, start);
-  int64_t prime = rit.prev_prime();
+  primesieve::iterator it1(stop, start);
+  int64_t prime = it1.prev_prime();
 
   if (prime <= start)
     return 0;
@@ -56,54 +55,59 @@ T P2_thread(T x,
   // The first iteration requires computing pi(x / prime)
   // using the prime counting function.
   int threads = 1;
-  int64_t xp = (int64_t)(x / prime);
+  uint64_t xp = (uint64_t)(x / prime);
   int64_t pi_xp = pi_noprint(xp, threads);
   T sum = pi_xp;
-  prime = rit.prev_prime();
+  prime = it1.prev_prime();
 
   // All other iterations compute pi(x / prime)
   // using a prime sieve.
-  primesieve::iterator it(xp, high);
-  int64_t p = it.next_prime();
+  primesieve::iterator it2(xp + 1, high);
+  it2.generate_next_primes();
 
   // \sum_{i = pi[start]+1}^{pi[stop]} pi(x / primes[i])
-  for (; prime > start; prime = rit.prev_prime())
+  for (; prime > start; prime = it1.prev_prime())
   {
-    xp = (int64_t)(x / prime);
-    for (; p <= xp; p = it.next_prime())
-      pi_xp++;
+    xp = (uint64_t)(x / prime);
+
+    for (; it2.primes_[it2.size_ - 1] <= xp; it2.generate_next_primes())
+      pi_xp += it2.size_ - it2.i_;
+    for (; it2.primes_[it2.i_] <= xp; it2.i_++)
+      pi_xp += 1;
+
     sum += pi_xp;
   }
 
   return sum;
 }
 
-/// P2(x, y) counts the numbers <= x that have exactly 2
+/// P2(x, a) counts the numbers <= x that have exactly 2
 /// prime factors each exceeding the a-th prime.
-/// Run time: O(n log log n), with n = x / y
+/// Run time: O(n log log n), with n = x / prime[a]
 /// Memory usage: O(n^(1/2))
 ///
 template <typename T>
 T P2_OpenMP(T x,
             int64_t y,
+            int64_t a,
             int threads,
             bool is_print)
 {
-  static_assert(is_signed<T>::value,
-                "T must be signed integer type");
+  ASSERT(a == pi_noprint(y, threads));
 
   if (x < 4)
     return 0;
 
   int64_t sqrtx = isqrt(x);
-  T a = pi_noprint(y, threads);
-  T b = pi_noprint(sqrtx, threads);
 
-  if (a >= b)
+  if (y >= sqrtx)
     return 0;
 
+  T b = pi_noprint(sqrtx, threads);
+
   // \sum_{i=a+1}^{b} -(i - 1)
   T sum = (a - 2) * (a + 1) / 2 - (b - 2) * (b + 1) / 2;
+  static_assert(port::is_signed<T>::value, "T must be signed integer type");
 
   int64_t xy = (int64_t)(x / max(y, 1));
   LoadBalancerP2 loadBalancer(x, xy, threads, is_print);
@@ -126,18 +130,21 @@ namespace primecount {
 
 int64_t P2(int64_t x,
            int64_t y,
+           int64_t a,
            int threads,
            bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== P2(x, y) ===");
     print_vars(x, y, threads);
+    time = get_time();
   }
 
-  double time = get_time();
-  int64_t sum = P2_OpenMP(x, y, threads, is_print);
+  int64_t sum = P2_OpenMP(x, y, a, threads, is_print);
 
   if (is_print)
     print("P2", sum, time);
@@ -149,18 +156,21 @@ int64_t P2(int64_t x,
 
 int128_t P2(int128_t x,
             int64_t y,
+            int64_t a,
             int threads,
             bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== P2(x, y) ===");
     print_vars(x, y, threads);
+    time = get_time();
   }
 
-  double time = get_time();
-  int128_t sum = P2_OpenMP(x, y, threads, is_print);
+  int128_t sum = P2_OpenMP(x, y, a, threads, is_print);
 
   if (is_print)
     print("P2", sum, time);
