@@ -7,7 +7,7 @@
 ///        however in many newer papers the ordinary leaves are named
 ///        S0. In primecount the ordinary leaves are named S1.
 ///
-/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2023 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -18,13 +18,12 @@
 #include <generate.hpp>
 #include <imath.hpp>
 #include <int128_t.hpp>
+#include <Vector.hpp>
 #include <print.hpp>
 #include <S.hpp>
 
 #include <stdint.h>
-#include <vector>
 
-using namespace std;
 using namespace primecount;
 
 namespace {
@@ -36,13 +35,13 @@ namespace {
 /// Algorithm For Computing pi(x)", arXiv:1503.01839, 6 March
 /// 2015.
 ///
-template <int MU, typename T, typename P>
+template <int MU, typename T, typename vect>
 T S1_thread(T x,
             int64_t y,
             uint64_t b,
             int64_t c,
             T square_free,
-            vector<P>& primes)
+            const vect& primes)
 {
   T s1 = 0;
 
@@ -67,7 +66,10 @@ X S1_OpenMP(X x,
             int64_t c,
             int threads)
 {
-  threads = ideal_num_threads(threads, y);
+  // These load balancing settings work well on my
+  // dual-socket AMD EPYC 7642 server with 192 CPU cores.
+  int64_t thread_threshold = (int64_t) 1e6;
+  threads = ideal_num_threads(y, threads, thread_threshold);
 
   auto primes = generate_primes<Y>(y);
   int64_t pi_y = primes.size() - 1;
@@ -93,14 +95,16 @@ int64_t S1(int64_t x,
            int threads,
            bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== S1(x, y) ===");
     print_vars(x, y, c, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   int64_t s1 = S1_OpenMP(x, y, c, threads);
 
   if (is_print)
@@ -117,18 +121,20 @@ int128_t S1(int128_t x,
             int threads,
             bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== S1(x, y) ===");
     print_vars(x, y, c, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   int128_t s1;
 
   // uses less memory
-  if (y <= numeric_limits<uint32_t>::max())
+  if (y <= port::numeric_limits<uint32_t>::max())
     s1 = S1_OpenMP(x, (uint32_t) y, c, threads);
   else
     s1 = S1_OpenMP(x, y, c, threads);
