@@ -13,7 +13,7 @@
 ///        method, Revista do DETUA, vol. 4, no. 6, March 2006,
 ///        pp. 759-768.
 ///
-/// Copyright (C) 2022 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2023 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -26,7 +26,7 @@
 #include <int128_t.hpp>
 #include <min.hpp>
 #include <imath.hpp>
-#include <pod_vector.hpp>
+#include <Vector.hpp>
 #include <print.hpp>
 #include <RelaxedAtomic.hpp>
 #include <StatusS2.hpp>
@@ -35,7 +35,6 @@
 #include <libdivide.h>
 #include <stdint.h>
 
-using std::numeric_limits;
 using namespace primecount;
 
 namespace {
@@ -158,14 +157,20 @@ T S2_easy_OpenMP(T x,
                  bool is_print)
 {
   // Initialize libdivide vector from primes vector
-  pod_vector<libdivide::branchfree_divider<uint64_t>> lprimes;
+  Vector<libdivide::branchfree_divider<uint64_t>> lprimes;
   lprimes.resize(primes.size());
   for (std::size_t i = 1; i < lprimes.size(); i++)
     lprimes[i] = primes[i];
 
   T sum = 0;
-  int64_t x13 = iroot<3>(x);
-  threads = ideal_num_threads(threads, x13, 1000);
+  int64_t x13 = iroot<3>(x); 
+
+  // These load balancing settings work well on my
+  // dual-socket AMD EPYC 7642 server with 192 CPU cores.
+  int64_t thread_threshold = 1000;
+  int max_threads = (int) std::pow(z, 1 / 4.0);
+  threads = std::min(threads, max_threads);
+  threads = ideal_num_threads(x13, threads, thread_threshold);
 
   StatusS2 status(x);
   PiTable pi(y, threads);
@@ -180,7 +185,7 @@ T S2_easy_OpenMP(T x,
     int64_t prime = primes[b];
     T xp = x / prime;
 
-    if (xp <= numeric_limits<uint64_t>::max())
+    if (xp <= port::numeric_limits<uint64_t>::max())
       sum += S2_easy_64(xp, y, z, b, prime, lprimes, pi);
     else
       sum += S2_easy_128(xp, y, z, b, prime, primes, pi);
@@ -204,14 +209,16 @@ int64_t S2_easy(int64_t x,
                 int threads,
                 bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== S2_easy(x, y) ===");
     print_vars(x, y, c, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   auto primes = generate_primes<uint32_t>(y);
   int64_t sum = S2_easy_OpenMP((uint64_t) x, y, z, c, primes, threads, is_print);
 
@@ -230,18 +237,20 @@ int128_t S2_easy(int128_t x,
                  int threads,
                  bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== S2_easy(x, y) ===");
     print_vars(x, y, c, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   int128_t sum;
 
   // uses less memory
-  if (y <= numeric_limits<uint32_t>::max())
+  if (y <= port::numeric_limits<uint32_t>::max())
   {
     auto primes = generate_primes<uint32_t>(y);
     sum = S2_easy_OpenMP((uint128_t) x, y, z, c, primes, threads, is_print);
