@@ -18,7 +18,7 @@
 ///        method, Revista do DETUA, vol. 4, no. 6, March 2006,
 ///        pp. 759-768.
 ///
-/// Copyright (C) 2021 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2024 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -40,7 +40,6 @@
 
 #include <stdint.h>
 
-using namespace std;
 using namespace primecount;
 
 namespace {
@@ -66,7 +65,7 @@ T S2_hard_thread(T x,
                  const Primes& primes,
                  const PiTable& pi,
                  const FactorTable& factor,
-                 ThreadSettings& thread)
+                 ThreadData& thread)
 {
   T sum = 0;
 
@@ -84,8 +83,8 @@ T S2_hard_thread(T x,
   if (min_b > max_b)
     return 0;
 
-  Sieve sieve(low, segment_size, max_b);
   auto phi = generate_phi(low, max_b, primes, pi);
+  Sieve sieve(low, segment_size, max_b);
   thread.init_finished();
 
   // Segmented sieve of Eratosthenes
@@ -198,21 +197,26 @@ T S2_hard_OpenMP(T x,
                  int threads,
                  bool is_print)
 {
+  // These load balancing settings work well on my
+  // dual-socket AMD EPYC 7642 server with 192 CPU cores.
   int64_t thread_threshold = 1 << 20;
-  threads = ideal_num_threads(threads, z, thread_threshold);
+  int max_threads = (int) std::pow(z, 1 / 3.7);
+  threads = std::min(threads, max_threads);
+  threads = ideal_num_threads(z, threads, thread_threshold);
+
   LoadBalancerS2 loadBalancer(x, z, s2_hard_approx, threads, is_print);
   int64_t max_prime = min(y, z / isqrt(y));
   PiTable pi(max_prime, threads);
 
   #pragma omp parallel num_threads(threads)
   {
-    ThreadSettings thread;
+    ThreadData thread;
 
     while (loadBalancer.get_work(thread))
     {
       // Unsigned integer division is usually slightly
       // faster than signed integer division
-      using UT = typename make_unsigned<T>::type;
+      using UT = typename port::make_unsigned<T>::type;
 
       thread.start_time();
       UT sum = S2_hard_thread((UT) x, y, z, c, primes, pi, factor, thread);
@@ -238,14 +242,16 @@ int64_t S2_hard(int64_t x,
                 int threads,
                 bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== S2_hard(x, y) ===");
     print_vars(x, y, c, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   FactorTable<uint16_t> factor(y, threads);
   int64_t max_prime = min(y, z / isqrt(y));
   auto primes = generate_primes<int32_t>(max_prime);
@@ -267,14 +273,16 @@ int128_t S2_hard(int128_t x,
                  int threads,
                  bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== S2_hard(x, y) ===");
     print_vars(x, y, c, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   int128_t sum;
 
   // uses less memory
