@@ -19,7 +19,7 @@
 ///        In-depth description of this algorithm:
 ///        https://github.com/kimwalisch/primecount/blob/master/doc/Easy-Special-Leaves.md
 ///
-/// Copyright (C) 2021 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2024 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -30,19 +30,16 @@
 #include <primecount-internal.hpp>
 #include <LoadBalancerAC.hpp>
 #include <fast_div.hpp>
-#include <for_atomic.hpp>
 #include <generate.hpp>
 #include <gourdon.hpp>
 #include <int128_t.hpp>
 #include <min.hpp>
 #include <imath.hpp>
 #include <print.hpp>
+#include <RelaxedAtomic.hpp>
 
 #include <stdint.h>
-#include <atomic>
-#include <vector>
 
-using namespace std;
 using namespace primecount;
 
 namespace {
@@ -213,8 +210,15 @@ T AC_OpenMP(T x,
   T sum = 0;
   int64_t x13 = iroot<3>(x);
   int64_t sqrtx = isqrt(x);
+  int64_t xy = x / y;
+  int64_t xz = x / z;
+
+  // These load balancing settings work well on my
+  // dual-socket AMD EPYC 7642 server with 192 CPU cores.
   int64_t thread_threshold = 1000;
-  threads = ideal_num_threads(threads, x13, thread_threshold);
+  int max_threads = (int) std::pow(xz, 1 / 3.7);
+  threads = std::min(threads, max_threads);
+  threads = ideal_num_threads(x13, threads, thread_threshold);
   LoadBalancerAC loadBalancer(sqrtx, y, threads, is_print);
 
   // PiTable's size = z because of the C1 formula.
@@ -225,9 +229,9 @@ T AC_OpenMP(T x,
 
   int64_t pi_y = pi[y];
   int64_t pi_sqrtz = pi[isqrt(z)];
-  int64_t pi_root3_xy = pi[iroot<3>(x / y)];
-  int64_t pi_root3_xz = pi[iroot<3>(x / z)];
-  atomic<int64_t> min_c1(max(k, pi_root3_xz) + 1);
+  int64_t pi_root3_xy = pi[iroot<3>(xy)];
+  int64_t pi_root3_xz = pi[iroot<3>(xz)];
+  RelaxedAtomic<int64_t> min_c1(max(k, pi_root3_xz) + 1);
 
   // In order to reduce the thread creation & destruction
   // overhead we reuse the same threads throughout the
@@ -239,15 +243,11 @@ T AC_OpenMP(T x,
   //
   #pragma omp parallel num_threads(threads) reduction(+: sum)
   {
-    // SegmentedPiTable is accessed very frequently.
-    // In order to get good performance it is important that
-    // SegmentedPiTable fits into the CPU's cache.
-    // Hence we use a small segment_size of x^(1/4).
-    SegmentedPiTable segmentedPi;
-    int64_t low, high;
-
     // C1 formula: pi[(x/z)^(1/3)] < b <= pi[pi_sqrtz]
-    for_atomic_inc(min_c1, b <= pi_sqrtz)
+    // There are very few iterations in this loop,
+    // hence the use of an atomic loop counter (min_c1)
+    // won't cause any scaling issues.
+    for (int64_t b = min_c1++; b <= pi_sqrtz; b = min_c1++)
     {
       int64_t prime = primes[b];
       T xp = x / prime;
@@ -258,38 +258,63 @@ T AC_OpenMP(T x,
       sum -= C1<-1>(xp, b, b, pi_y, 1, min_m, max_m, primes, pi);
     }
 
-    // for (low = 0; low < sqrt; low += segment_size)
-    while (loadBalancer.get_work(low, high))
+    // SegmentedPiTable is accessed very frequently.
+    // In order to get good performance it is important that
+    // SegmentedPiTable fits into the CPU's cache.
+    // Hence we use a small segment_size of x^(1/4).
+    SegmentedPiTable segmentedPi;
+    ThreadDataAC thread;
+
+    // for (low = 0; low < sqrt(x); low += segment_size)
+    while (loadBalancer.get_work(thread))
     {
-      // Current segment [low, high[
-      segmentedPi.init(low, high);
-      T xlow = x / max(low, 1);
-      T xhigh = x / high;
+      int64_t low = thread.low;
+      int64_t segment_size = thread.segment_size;
+      int64_t limit = low + thread.segments * segment_size;
+      limit = std::min(limit, sqrtx);
 
-      int64_t min_c2 = max(k, pi_root3_xy);
-      min_c2 = max(min_c2, pi_sqrtz);
-      min_c2 = max(min_c2, pi[isqrt(low)]);
-      min_c2 = max(min_c2, pi[min(xhigh / y, x_star)]);
-      min_c2 += 1;
+      for (; low < limit; low += segment_size)
+      {
+        // Current segment [low, high[
+        int64_t high = low + segment_size;
+        high = std::min(high, sqrtx);
+        segmentedPi.init(low, high);
 
-      int64_t min_a = min(xhigh / high, x13);
-      min_a = pi[max(x_star, min_a)] + 1;
-    
-      // Upper bound of A & C2 formulas:
-      // x / (p * q) >= low
-      // p * next_prime(p) <= x / low
-      // p <= sqrt(x / low)
-      T sqrt_xlow = isqrt(xlow);
-      int64_t max_c2 = pi[min(sqrt_xlow, x_star)];
-      int64_t max_a = pi[min(sqrt_xlow, x13)];
+        // We measure the thread computation time excluding the
+        // first expensive initialization of the segmentedPi
+        // lookup table. If the thread computation time is close
+        // to 0 then we increase the number of segments in the
+        // loadBalancer which should improve performance.
+        if (low == thread.low)
+          thread.secs = get_time();
 
-      // C2 formula: pi[sqrt(z)] < b <= pi[x_star]
-      for (int64_t b = min_c2; b <= max_c2; b++)
-        sum += C2(x, xlow, xhigh, y, b, primes, pi, segmentedPi);
+        T xlow = x / max(low, 1);
+        T xhigh = x / high;
+        int64_t min_c2 = max(k, pi_root3_xy);
+        min_c2 = max(min_c2, pi_sqrtz);
+        min_c2 = max(min_c2, pi[isqrt(low)]);
+        min_c2 = max(min_c2, pi[min(xhigh / y, x_star)]);
+        min_c2 += 1;
 
-      // A formula: pi[x_star] < b <= pi[x13]
-      for (int64_t b = min_a; b <= max_a; b++)
-        sum += A(x, xlow, xhigh, y, b, primes, pi, segmentedPi);
+        int64_t min_a = min(xhigh / high, x13);
+        min_a = pi[max(x_star, min_a)] + 1;
+      
+        // Upper bound of A & C2 formulas:
+        // x / (p * q) >= low
+        // p * next_prime(p) <= x / low
+        // p <= sqrt(x / low)
+        T sqrt_xlow = isqrt(xlow);
+        int64_t max_c2 = pi[min(sqrt_xlow, x_star)];
+        int64_t max_a = pi[min(sqrt_xlow, x13)];
+
+        // C2 formula: pi[sqrt(z)] < b <= pi[x_star]
+        for (int64_t b = min_c2; b <= max_c2; b++)
+          sum += C2(x, xlow, xhigh, y, b, primes, pi, segmentedPi);
+
+        // A formula: pi[x_star] < b <= pi[x13]
+        for (int64_t b = min_a; b <= max_a; b++)
+          sum += A(x, xlow, xhigh, y, b, primes, pi, segmentedPi);
+      }
     }
   }
 
@@ -307,14 +332,16 @@ int64_t AC(int64_t x,
            int threads,
            bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== AC(x, y) ===");
     print_gourdon_vars(x, y, z, k, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   int64_t x_star = get_x_star_gourdon(x, y);
   int64_t max_c_prime = y;
   int64_t max_a_prime = (int64_t) isqrt(x / x_star);
@@ -338,14 +365,16 @@ int128_t AC(int128_t x,
             int threads,
             bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== AC(x, y) ===");
     print_gourdon_vars(x, y, z, k, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   int64_t x_star = get_x_star_gourdon(x, y);
   int64_t max_c_prime = y;
   int64_t max_a_prime = (int64_t) isqrt(x / x_star);
@@ -353,7 +382,7 @@ int128_t AC(int128_t x,
   int128_t sum;
 
   // uses less memory
-  if (max_prime <= numeric_limits<uint32_t>::max())
+  if (max_prime <= port::numeric_limits<uint32_t>::max())
   {
     auto primes = generate_primes<uint32_t>(max_prime);
     sum = AC_OpenMP((uint128_t) x, y, z, k, x_star, max_a_prime, primes, threads, is_print);
