@@ -13,7 +13,7 @@
 ///        z < x^(1/2). Also the small constant is named k instead
 ///        of c.
 ///
-/// Copyright (C) 2021 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2024 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -26,11 +26,10 @@
 #include <imath.hpp>
 #include <int128_t.hpp>
 #include <print.hpp>
+#include <Vector.hpp>
 
 #include <stdint.h>
-#include <vector>
 
-using namespace std;
 using namespace primecount;
 
 namespace {
@@ -48,7 +47,7 @@ T Phi0_thread(T x,
               uint64_t b,
               int64_t k,
               T square_free,
-              vector<P>& primes)
+              const Vector<P>& primes)
 {
   T phi0 = 0;
 
@@ -74,7 +73,10 @@ X Phi0_OpenMP(X x,
               int64_t k,
               int threads)
 {
-  threads = ideal_num_threads(threads, y);
+  // These load balancing settings work well on my
+  // dual-socket AMD EPYC 7642 server with 192 CPU cores.
+  int64_t thread_threshold = (int64_t) 1e6;
+  threads = ideal_num_threads(y, threads, thread_threshold);
 
   auto primes = generate_primes<Y>(y);
   int64_t pi_y = primes.size() - 1;
@@ -101,14 +103,16 @@ int64_t Phi0(int64_t x,
              int threads,
              bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== Phi0(x, y) ===");
     print_gourdon_vars(x, y, z, k, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   int64_t phi0 = Phi0_OpenMP(x, y, z, k, threads);
 
   if (is_print)
@@ -126,18 +130,20 @@ int128_t Phi0(int128_t x,
               int threads,
               bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== Phi0(x, y) ===");
     print_gourdon_vars(x, y, z, k, threads);
+    time = get_time();
   }
 
-  double time = get_time();
   int128_t phi0;
 
   // uses less memory
-  if (y <= numeric_limits<uint32_t>::max())
+  if (y <= port::numeric_limits<uint32_t>::max())
     phi0 = Phi0_OpenMP(x, (uint32_t) y, z, k, threads);
   else
     phi0 = Phi0_OpenMP(x, y, z, k, threads);
