@@ -17,7 +17,7 @@
 ///        pi(x) = pi(y) + S1(x, a) + S2(x, a) - 1 - P2(x, a)
 ///        with y = x^(1/3), a = pi(y)
 ///
-/// Copyright (C) 2020 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2023 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -30,12 +30,11 @@
 #include <imath.hpp>
 #include <PhiTiny.hpp>
 #include <print.hpp>
+#include <Vector.hpp>
 #include <S.hpp>
 
 #include <stdint.h>
-#include <vector>
 
-using namespace std;
 using namespace primecount;
 
 namespace {
@@ -44,25 +43,28 @@ namespace {
 int64_t S2(int64_t x,
            int64_t y,
            int64_t c,
-           const vector<int32_t>& primes,
-           const vector<int32_t>& lpf,
-           const vector<int32_t>& mu,
+           const Vector<int32_t>& primes,
+           const Vector<int32_t>& lpf,
+           const Vector<int32_t>& mu,
            bool is_print)
 {
+  double time;
+
   if (is_print)
   {
     print("");
     print("=== S2(x, y) ===");
+    time = get_time();
   }
 
-  double time = get_time();
   int64_t limit = x / y;
   int64_t segment_size = Sieve::get_segment_size(isqrt(limit));
   int64_t low = 0;
 
   Sieve sieve(low, segment_size, primes.size());
-  vector<int32_t> pi = generate_pi(y);
-  vector<int64_t> phi(primes.size(), 0);
+  auto pi = generate_pi(y);
+  Vector<int64_t> phi(primes.size());
+  std::fill(phi.begin(), phi.end(), 0);
 
   int64_t s2 = 0;
   int64_t pi_sqrty = pi[isqrt(y)];
@@ -154,7 +156,7 @@ int64_t pi_lmo5(int64_t x, bool is_print)
   if (x < 2)
     return 0;
 
-  bool threads = 1;
+  int threads = 1;
   double alpha = get_alpha_lmo(x);
   int64_t x13 = iroot<3>(x);
   int64_t y = (int64_t) (x13 * alpha);
@@ -169,12 +171,12 @@ int64_t pi_lmo5(int64_t x, bool is_print)
     print(x, y, z, c, threads);
   }
 
-  int64_t p2 = P2(x, y, threads, is_print);
   auto primes = generate_primes<int32_t>(y);
   auto lpf = generate_lpf(y);
   auto mu = generate_moebius(y);
 
   int64_t pi_y = primes.size() - 1;
+  int64_t p2 = P2(x, y, pi_y, threads);
   int64_t s1 = S1(x, y, c, threads, is_print);
   int64_t s2 = S2(x, y, c, primes, lpf, mu, is_print);
   int64_t phi = s1 + s2;
